#
# lacrosse.py
#
# Copyright (C) 2010 Tim Wentzlau
#
# This file is part of the XBMC Weather Station script
# 
# The Weather Station script is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# XBMC Weather station script is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with EventGhost; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA


TEMPOUT=0
TEMPIN=1
WINDCHILL=2
DEWPOINT=3
WINDSPEED=4
WINDDIRECTION=5
PRESSURE=6
HUMIDITY=7
RAINTOTAL=8
RAINHOUR=9
RAINPERIOD=10
MAXFIELDS=11

histLogMap=[[TEMPOUT,4],[TEMPIN,3],[DEWPOINT,5],[PRESSURE,13],[WINDSPEED,8],[HUMIDITY,7],[WINDDIRECTION,9],[WINDCHILL,0],[RAINTOTAL,12],[RAINHOUR,0],[RAINPERIOD,0]]
log2300Map=[[TEMPOUT,4],[TEMPIN,3],[DEWPOINT,8],[PRESSURE,15],[WINDSPEED,8],[HUMIDITY,7],[WINDDIRECTION,9],[WINDCHILL,0],[RAINTOTAL,14],[RAINHOUR,12],[RAINPERIOD,13]]

MAX=0
MIN=1
VALUE=2
DATA=3

import os

resPath=os.path.dirname(os.path.realpath(__file__))+"\\open2300\\"

class LacrosseStations:
	
	data=[]
	
	period=12
	def __init__(self,period):
		f=0
		while f<MAXFIELDS:
			self.data.append([0,0,0,[]])
			f+=1
		self.period=period
		#self.loadData()
	
	def getDataMin(self,data):
		res=100000000
		for dat in data:
			d=dat[1]
			if d<res: 
				res=d
		return res

	def getDataMax(self,data):
		res=-100000000
		for dat in data:
			d=dat[1]
			if d>res: 
				res=d
		return res

	def getValue(self,set):
		return self.data[set][VALUE]
		
	def getDataSet(self,set):
		return self.data[set][DATA]
	
	def getMin(self,set):
		return self.data[set][MIN]
	
	def getMax(self,set):
		return self.data[set][MAX]
	
	def sizeData(self,list):
		if (len(list)>24):
			return list[len(list)-24:len(list)]	
		else:
			return list
		
	def loadData(self):
		#import subprocess
		#subprocess.check_call('"'+resPath+'histlog2300.exe"',"histlog.log")
		
		histLog=resPath+"histlog.log"
		open2300Log=resPath+"log2300.log"
		logMap=histLogMap
		try:
			fin=open(histLog,"r")
			for line in fin:
				lineData=line.split(" ")
				t=lineData[2]
				t=t[0:5]
			
			
				for f in logMap: 
					fd=f[0]
					df=f[1]
					if df>0:
						if (float(lineData[4])>50) and (len(self.data[fd][DATA])>0):
							self.data[fd][DATA].append([t,self.data[fd][DATA][-1]][1]) 
						else:
							self.data[fd][DATA].append([t,float(lineData[df])])
						if(fd==WINDDIRECTION):
							self.data[fd][DATA][-1][1]=-self.data[fd][DATA][-1][1]
					
			fin.close()
			
			i=0
			while i<len(self.data[TEMPOUT][DATA]):
				dt=self.data[TEMPOUT][DATA][i]
				dw=self.data[WINDSPEED][DATA][i]
				time=dt[0]
				t=dt[1]
				v=dw[1]
				if v>2:
					chill = round(13.12 + (0.6215 * t) - (13.96 * v ** .16) + (.4867 * t * v ** .16),2)
				else:
					chill=t
				self.data[WINDCHILL][DATA].append([t,chill])
				i+=1
			
			if len(self.data[RAINTOTAL][DATA])>1:
				i=1
				total=0
				while i<len(self.data[RAINTOTAL][DATA]):
					time=self.data[RAINTOTAL][DATA][i][0]
					s=self.data[RAINTOTAL][DATA][i-1][1]
					t=self.data[RAINTOTAL][DATA][i][1]
					total+=t-s
					self.data[RAINHOUR][DATA].append([time,t-s])
					self.data[RAINPERIOD][DATA].append([time,total])
					i+=1
				
			for f in logMap: 
				fd=f[0]
				self.data[fd][DATA]=self.sizeData(self.data[fd][DATA])
				self.data[fd][MIN]=self.getDataMin(self.data[fd][DATA])
				self.data[fd][MAX]=self.getDataMax(self.data[fd][DATA])
				self.data[fd][VALUE]=self.data[fd][DATA][-1][1]
			
			
			
		except IOError:
			print "can not open histlog.log file"
			
		try:	
			fin=open(open2300Log,"r")
			lastLine=""
			for line in fin:
				lastLine=line
			
			fin.close()
			
			if len(lastLine)>0:
				lineData=line.split(" ")
				t=lineData[2]
				t=t[0:5]
				
				for f in log2300Map: 
					fd=f[0]
					df=f[1]
					if df>0:
						if (float(lineData[4])>50) and (len(self.data[fd][DATA])>0):
							self.data[fd][VALUE]=self.data[fd][DATA][-1][1] 
						else:
							self.data[fd][VALUE]=float(lineData[df])
						if(fd==WINDDIRECTION):
							self.data[fd][DATA][-1][1]=-self.data[fd][DATA][-1][1]
		
				v=self.data[WINDSPEED][VALUE]
				t=self.data[TEMPOUT][VALUE]
				if v>2:
					chill = round(13.12 + (0.6215 * t) - (13.96 * v ** .16) + (.4867 * t * v ** .16),2)
				else:
					chill=t
				
				self.data[WINDCHILL][VALUE]=chill
				
		except IOError:
			print "can not open log2300.log file"
			
		